# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from command import Command, _
from ionas.request import Entry, XmlNode
from datetime import datetime, time
from fal_info import Results


MASTER = "master"
SLAVE = "slave"
SYSTEM = "system"
ENABLE = "enable"
HOST = "host"
PASSWD = "passwd"
WATCH = "watch"
DEADTIME = "deadtime"
TIMES = "times"
SCHEDULE = "schedule"
DAY = "day"
TIME = "time"

DAY_LIST = ('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat')


class Clone(Command):
    try:
        from fal_info import FAL_CLONE_PATH as COMMAND
    except:
        COMMAND_NAME = 'fal_clone.py'

    TYPE_QUERY = 'query'
    TYPE_APPLY = 'apply'
    TYPE_BACKUP = 'backup'
    TYPE_RESTORE = 'restore'
    TYPE_SWITCH = 'switch'
    TYPE_STOP = 'stop'

    def _query(self):
        entry = Entry(self.TYPE_QUERY, """<clone />""")
        return self.request(entry)

    def _apply(self, config={}):
        """
        config={
            "master": {
                "enable": True,
                "slave": {
                    "host": "192.168.11.22",
                    "passwd": "password",
                },
            },
            "slave": {
                "watch": {
                    "enable": True,
                    "daedtime": 10,
                    "master": {
                        "host": "192.168.11.33",
                    },
                },
            },
            "system": {
                "times": 0,
                "schedule": {
                    "enable": True,
                    "day": {  #(実行, フル)
                        "sun": {
                            "enable": True,
                            "full": True,
                        },
                        "mon": {...},
                        "tue": {...},
                        "wed": {...},
                        "thu": {...},
                        "fri": {...},
                        "sat": {...},
                    },
                    "time": datetime.time(12, 12),
                },
            },
        }
        """
        xmls = {MASTER: "", SLAVE: "", SYSTEM: ""}
        if MASTER in config:
            c = config[MASTER]
            enable = c[ENABLE] if isinstance(c.get(ENABLE), bool) else False
            host = c.get(SLAVE, {}).get(HOST, "")
            passwd = c.get(SLAVE, {}).get(PASSWD, "")
            xmls[MASTER] = """<master enable="%(enable)s">
                <slave>
                    <host>%(host)s</host>
                    <passwd>%(passwd)s</passwd>
                </slave>
            </master>""" % {
                "enable": str(enable).lower(),
                "host": host,
                "passwd": passwd,
            }
        if SLAVE in config:
            c = config[SLAVE]
            wenable = c.get(WATCH, {}).get(ENABLE)
            wenable = wenable if isinstance(wenable, bool) else False
            wdeadtime = c.get(WATCH, {}).get(DEADTIME, 10)
            host = c.get(WATCH, {}).get(MASTER, {}).get(HOST, "")
            xmls[SLAVE] = """<slave>
                <watch enable="%(enable)s">
                    <deadtime>%(deadtime)s</deadtime>
                    <master>
                        <host>%(host)s</host>
                    </master>
                </watch>
            </slave>""" % {
                "enable": str(wenable).lower(),
                "deadtime": wdeadtime,
                "host": host,
            }
        if SYSTEM in config:
            c = config[SYSTEM]
            senable = c.get(SCHEDULE, {}).get(ENABLE)
            senable = senable if isinstance(senable, bool) else False
            sdays = c.get(SCHEDULE, {}).get(DAY, [])
            stime = c.get(SCHEDULE, {}).get(TIME, time(0, 0))  # FIXME time
            xmls[SYSTEM] = """<system>
                <times>%(stimes)s</times>
                <schedule enable="%(senable)s">
                    %(sday)s
                    <time>
                        <hour>%(shour)s</hour>
                        <minute>%(sminute)s</minute>
                    </time>
                </schedule>
            </system>""" % {
                "senable": str(senable).lower(),
                "sday": "".join(["""<day full="%(full)s">%(day)s</day>""" % {
                        'day': day.capitalize(),
                        'full': str(prm['full']).lower(),
                } for day, prm in sdays.iteritems() if prm['enable']]),
                "shour": stime.hour,
                "sminute": stime.minute,
                'stimes': int(c.get(TIMES, 0)),
            }
        entry = Entry(self.TYPE_APPLY, """<clone>
            %(master)s
            %(slave)s
            %(system)s
        </clone>""" % {
            "master": xmls[MASTER],
            "slave": xmls[SLAVE],
            "system": xmls[SYSTEM],
        })
        return self.request(entry)

    def _backup(self, config=True, share=True, full=False):
        entry = Entry(
            self.TYPE_BACKUP,
            """<clone>
                <config enable="%(config)s" />
                <share enable="%(share)s" full="%(full)s" />
            </clone>""" % {
                'config': str(config).lower(),
                'share': str(share).lower(),
                'full': str(full).lower(),
            })
        return self.request(entry)

    def _restore(self, config=True, share=True, generation=None):
        entry = Entry(
            self.TYPE_RESTORE,
            """<clone>
                <config enable="%(config)s" />
                <share enable="%(share)s"%(generation)s />
            </clone>""" % {
                'config': str(config).lower(),
                'share': str(share).lower(),
                'generation': ' generation="%s"' % generation if generation else '',
            })
        return self.request(entry)

    def _switch(self):
        entry = Entry(self.TYPE_SWITCH, """<clone />""")
        return self.request(entry)

    def _stop(self):
        entry = Entry(self.TYPE_STOP, """<clone />""")
        return self.request(entry)

    #
    # 外部用
    #

    def query(self, to_dict=True):
        def _to_dict(entry):
            dict_ = {}
            try:
                cnd = entry.get()  # clone node
                def _is_enabled(xpath, default=False):
                    try:
                        return cnd.get_node(xpath).is_enabled()
                    except:
                        return default

                def _is(xpath, name, default=False):
                    try:
                        return cnd.get_node(xpath).is_set_to_true(name)
                    except:
                        return default

                dict_ = {
                    "master": {
                        "enable": _is_enabled('master'),
                        "slave": {
                            "host": cnd.get_value('master/slave/host', ''),
                            "passwd": cnd.get_value('master/slave/passwd', ''),
                        },
                    },
                    "slave": {
                        #"switch": cnd.get_node('slave').is_set_to_true('switch'),
                        "switch": _is('slave', 'switch'),
                        "watch": {
                            "enable": _is_enabled('slave/watch'),
                            "deadtime": int(cnd.get_value('slave/watch/deadtime', '10')),
                            "master": {
                                "host": cnd.get_value('slave/watch/master/host', ''),
                            },
                        },
                    },
                    "system": {
                        #"restore": cnd.get_node('system').is_set_to_true('restore'),
                        "restore": _is('system', 'restore'),
                        "times": int(cnd.get_value('system/times', '0')),
                        "schedule": {
                            "enable": _is_enabled('system/schedule'),
                            "day": dict([(day, {
                                    'enable': bool(cnd.get_node_list('system/schedule/day[text()="%s"]' % day.capitalize())),
                                    'full': _is('system/schedule/day[text()="%s"]' % day.capitalize(), 'full'),
                                    }) for day in ('sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat')]),
                            #"day": {
                            #    "sun": {
                            #        'enable': bool(cnd.get_node_list('system/schedule/day[text()="Sun"]')),
                            #        'full': _is('system/schedule/day[text()="Sun"]', 'full'),
                            #    },
                            #    ...
                            #},
                            "time": time(
                                int(cnd.get_value('system/schedule/time/hour', 0)),
                                int(cnd.get_value('system/schedule/time/minute', 0))),
                        },
                    },
                    "saveconfig": {
                        "restore": {
                            "hostname": cnd.get_value('saveconfig/config[@for="restore"]/hostname', ''),
                            "timestamp": cnd.get_value('saveconfig/config[@for="restore"]/timestamp', ''),
                            "ipaddr": {
                                "lan1": {
                                    "ipv4": cnd.get_value('saveconfig/config[@for="restore"]/ipaddr[@lan="1"][@ipv="4"]'),
                                    "ipv6": cnd.get_value('saveconfig/config[@for="restore"]/ipaddr[@lan="1"][@ipv="6"]'),
                                },
                                "lan2": {
                                    "ipv4": cnd.get_value('saveconfig/config[@for="restore"]/ipaddr[@lan="2"][@ipv="4"]'),
                                    "ipv6": cnd.get_value('saveconfig/config[@for="restore"]/ipaddr[@lan="2"][@ipv="6"]'),
                                },
                            },
                        },
                        "switch": {
                            "hostname": cnd.get_value('saveconfig/config[@for="switch"]/hostname', ''),
                            "timestamp": cnd.get_value('saveconfig/config[@for="switch"]/timestamp', ''),
                            "ipaddr": {
                                "lan1": {
                                    "ipv4": cnd.get_value('saveconfig/config[@for="switch"]/ipaddr[@lan="1"][@ipv="4"]'),
                                    "ipv6": cnd.get_value('saveconfig/config[@for="switch"]/ipaddr[@lan="1"][@ipv="6"]'),
                                },
                                "lan2": {
                                    "ipv4": cnd.get_value('saveconfig/config[@for="switch"]/ipaddr[@lan="2"][@ipv="4"]'),
                                    "ipv6": cnd.get_value('saveconfig/config[@for="switch"]/ipaddr[@lan="2"][@ipv="6"]'),
                                },
                            },
                        },
                    },
                    'generationlist': [nd.get_value('id') for nd in cnd.get_node_list('generationlist/generation')]
                }
            except:
                pass
            return dict_

        entry = self._query()
        return _to_dict(entry) if to_dict else entry

    def apply(self, config={}, log=True, to_bool=True, default_messages={}):

        _default_messages = {
            Results.OK: ['MSG_HCLONE_CHANGE', {}],
            '11': ['ERR_HCLONE_CHANGE_USB_DISCONNECT', {}],
            '12': ['ERR_HCLONE_CHANGE_USB_NOT_SHARE', {}],
            '13': ['ERR_HCLONE_CHANGE_USB_NOT_EXT4', {}],
            '21': ['ERR_HCLONE_MASTER_DEAD', {}],
            None: ['ERR_HCLONE_CHANGE', {}],
        }
        _default_messages.update(default_messages)

        entry = self._apply(config=config)
        result = entry.get_result()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK]) if to_bool else entry

    def backup(self,
            config=True,
            share=True,
            full=False,
            log=True, to_bool=True, default_messages={}):
        _default_messages = {
            Results.OK: ['MSG_HCLONE_SYSBACKUP_START', {}],
            "1": ['ERR_HCLONE_SYSBACKUP_USB_DISCONNECT', {}],
            "2": ['ERR_HCLONE_SYSBACKUP_USB_NOT_SHARE', {}],
            "3": ['ERR_HCLONE_SYSBACKUP_USB_NOT_EXT4', {}],
            None: ['ERR_HCLONE_SYSBACKUP', {}],
        }
        _default_messages.update(default_messages)

        entry = self._backup(config=config, share=share, full=full)
        result = entry.get_result()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK]) if to_bool else entry

    def restore(self,
            config=True,
            share=True,
            generation=None,
            log=True, to_bool=True, default_messages={}):
        _default_messages = {
            Results.OK: ['MSG_HCLONE_SYSRESTORE_START', {}],  # "0"
            "1": ['ERR_HCLONE_SYSRESTORE_USB', {}],
            None: ['ERR_HCLONE_SYSRESTORE', {}],
        }
        _default_messages.update(default_messages)

        entry = self._restore(config=config, share=share, generation=generation)
        result = entry.get_result()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK]) if to_bool else entry

    def switch(self, log=True, to_bool=True, default_messages={}):
        _default_messages = {
            Results.OK: ['MSG_HCLONE_SWITCH_COMPLETE', {}],
            None: ['ERR_HCLONE_SWITCH', {}],
        }
        _default_messages.update(default_messages)

        entry = self._switch()
        result = entry.get_result()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK] if to_bool else entry)

    def stop(self, log=True, to_bool=True, default_messages={}):
        _default_messages = {  # FIXME msg
            Results.OK: ['MSG_HCLONE_SYSBACKUP_STOP', {}],
            None: ['ERR_HCLONE_SYSBACKUP_STOP', {}],
        }
        _default_messages.update(default_messages)

        entry = self._stop()
        result = entry.get_result()

        message = _default_messages.get(result, _default_messages[None])
        self._set_message(message[0], arglist=message[1], log=log)

        return (result in [Results.OK] if to_bool else entry)

if __name__ == '__main__':
    c = Clone()
    print c.apply(config={
        "master": {
            "enable": True,
            "slave": {
                "host": "192.168.11.22",
                "passwd": "ioIO10!!",
            },
        },
        "slave": {
            "watch": {
                "enable": True,
                "deadtime": 30,
                "master": {
                    "host": "192.168.11.33",
                },
            },
        },
        "system": {
            "enable": True,
            "schedule": {
                "enable": True,
                "day": {
                    "wed": {
                        "enable": True,
                        "full": True,
                    },
                    "fri": {
                        "enable": False,
                        "full": False,
                    },
                },
                "time": time(12, 12),
            },
        },
    })
    print c.get_message()
    print c.query()
    print "backup: %s" % c.backup(full=True)
    print "restore: %s" % c.restore()
    print "switch: %s" % c.switch()
