# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.forms.fields.extras.enable import EnableField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.landiskpass import LanDiskPassField
from project.fal.clone import Clone
from project.forms.fields.select import SelectField
import re


class SelectBackupGenerationField(SelectField):

    def _choice(self, yyyyMMddHHmmss):
        import re
        m = re.match(r'(?P<year>[0-9]{4})(?P<month>[0-9]{2})(?P<day>[0-9]{2})(?P<hour>[0-9]{2})(?P<min>[0-9]{2})(?P<sec>[0-9]{2})', yyyyMMddHHmmss)
        if m:
            return (yyyyMMddHHmmss, u'%s/%s/%s %s:%s:%s' % (
                    m.group('year'),
                    m.group('month'),
                    m.group('day'),
                    m.group('hour'),
                    m.group('min'),
                    m.group('sec')))
        else:
            return (yyyyMMddHHmmss, yyyyMMddHHmmss)

    def _generations(self):
        try:
            return reversed(sorted(Clone().query()['generationlist']))
        except:
            return []

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = [self._choice(g) for g in self._generations()]
        if len(kwargs['choices']) == 0:
            kwargs['choices'] = [('', _(u"LanDiskClone: Saved Data is not exists."))]
        super(SelectBackupGenerationField, self).__init__(*args, **kwargs)


class Form(forms.Form):
    def __init__(self, *args, **kwargs):
        super(Form, self).__init__(*args, **kwargs)
        self.fields['generation'] = SelectBackupGenerationField(label=_(u'LanDiskClone: Restore Generation'))

class View(FuncView):


    views = ['execute']

#    def _is_ok(self):
#        try:
#            return Clone().query()['system']['restore']
#        except:
#            return False

    def _info(self):
        try:
            data = Clone().query()
            return {
                'hostname': data['saveconfig']['restore']['hostname'],
                'timestamp': data['saveconfig']['restore']['timestamp'],
                'lan1': {
                    'ipv4': data['saveconfig']['restore']['ipaddr']['lan1']['ipv4'],
                    'ipv6': data['saveconfig']['restore']['ipaddr']['lan1']['ipv6'],
                },
                'lan2': {
                    'ipv4': data['saveconfig']['restore']['ipaddr']['lan2']['ipv4'],
                    'ipv6': data['saveconfig']['restore']['ipaddr']['lan2']['ipv6'],
                },
            }
        except:
            return {}

    def view(self, request):
        form = Form()
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'info': self._info(),
                'disabled': False,
#                'ok': self._is_ok(),
            },
        }

    def view_execute(self, request):
        disabled = False
        with self.processing() as p:
            form = Form(data=request.POST)
            p.break_if_state_change()

            if not self._info()['hostname']:
                self.set_message(_(u"LanDiskClone: System Backup Data is not found."))
            elif form.is_valid():
                clone = Clone()
                p.result(clone.restore(
                        generation=form.cleaned_data['generation'],
                        log=False))
                self.set_message(clone.get_message())
                if p.result():
                    self.set_message(_(u"LanDiskClone: Please close browser."))
                    self.set_message(_(u"LanDiskClone: Please open top page when stop blink LED."))
                    disabled = True
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {
                'form': form,
                'info': self._info(),
                'disabled': disabled,
#                'ok': self._is_ok(),
            },
        }
