# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta
from django.utils.translation import ugettext as _
from django import forms
from django.core.exceptions import ValidationError
from project.view import FuncView
from project.forms.fields.extras.enable import EnableField
from project.forms.fields.extras.fqdn import FQDNField
from project.forms.fields.extras.landiskpass import LanDiskPassField
from project.forms.fields.select import SelectField
from project.fal.clone import Clone


class View(FuncView):

    class Form(forms.Form):
        enable = EnableField(
                label=_(u'LanDiskClone: Watch Master.'))
        deadtime = SelectField(
                label=_(u'LanDiskClone: Dead Time'),
                choices=[
                        (10, 10),
                        (20, 20),
                        (30, 30),
                        (40, 40),
                        (50, 50),
                        (60, 60)],
                required=False)
        master = FQDNField(
                label=_(u'LanDiskClone: Master'),
                required=False)

        def clean(self):
            data = self.cleaned_data
            if self.cleaned_data.get('enable', None):
                if self.cleaned_data.get('master', None) in [""]:
                    raise ValidationError("%s: %s" % (
                            _(u'LanDiskClone: Master'),
                            _(u'This field is required.')))
                if self.cleaned_data.get('deadtime', None) in [""]:
                    raise ValidationError("%s: %s" % (
                            _(u'LanDiskClone: Dead Time'),
                            _(u'This field is required.')))
            return data

    views = ['apply']

    def view(self, request):
        clone = Clone()
        config = clone.query()
        form = self.Form(initial={
            'enable': config['slave']['watch']['enable'],
            'deadtime': config['slave']['watch']['deadtime'],
            'master': config['slave']['watch']['master']['host'],
        })
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form},
        }

    def view_apply(self, request):
        with self.processing() as p:
            form = self.Form(data=request.POST)
            p.break_if_state_change()

            if form.is_valid():
                clone = Clone()
                config = {
                    'slave': {
                        'watch': {
                            'enable': form.cleaned_data['enable'],
                            'deadtime': form.cleaned_data['deadtime'],
                            'master': {
                                'host': form.cleaned_data['master'],
                            },
                        },
                    },
                }
                p.result(clone.apply(config=config))
                self.set_message(clone.get_message())
        return {
            self.KEY_TEMPLATE: self.TEMPLATE,
            self.KEY_TEMPLATE_DIC: {'form': form},
        }
